<?php

use Omnipay\Omnipay;

App::uses('PaypalLib', 'EvPaypal.Lib');
App::uses('GatewayBaseComponent', 'EvTransactions.Controller/Component');
App::uses('GatewayInterface', 'EvTransactions.Lib');
App::uses('CakeText', 'Utility');

class StripeComponent extends GatewayBaseComponent implements GatewayInterface {

	protected $_controller = null;

	protected $_api = null;

	public $Transaction = null;

	public function initialize(Controller $controller) {
		parent::initialize($controller);
		$this->_controller = $controller;
		$this->_api = Omnipay::create('Stripe');
		$this->Transaction = ClassRegistry::init('Transactions.Transaction');

		// Load the Stripe config here rather than the `setup()` method as we need these settings in
		// `beforeRender()` which is called before setup.
		if (Configure::read('app.environment') === 'PRODUCTION') {
			$this->_config = Configure::read('EvTransactions.stripe.live');
		} else {
			$this->_config = Configure::read('EvTransactions.stripe.dev');
		}
	}

	/**
	 * init
	 *
	 * Use the function setup and connection / settings that need setting up in the gateway
	 */
	public function setup() {
		// Set the API key.
		$this->_api->setApiKey($this->_config['secret_key']);
	}

	/**
	 * Before render. Sets the publishable key to a view variable.
	 * @param  Controller $controller
	 * @return void
	 */
	public function beforeRender(Controller $controller) {
		$controller->set('stripePublishableKey', $this->_config['publishable_key']);
	}

	/**
	 * setupPayment
	 *
	 * Use this function setup the actual payment, i.e. setup the basket, the amount to take etc...
	 *
	 * @param int - transaction id we have created
	 * @param array $return - array with 2 keys of 'return', 'cancel'. Containing either a link or router array for redirecting user
	 * @param array $model - array with two keys of 'model' and 'model_id', used to link transactions polymorphically to other model items
	 * @param float|array $amount - amount of monies to take (I GOT YOUR MONIESSSSSS), or array of 'amount' and 'currency' to change currencies (will take default from config)
	 * @param array $items - multidimenisional array break down of the transaction items, required elements are 'description' and 'amount'
	 * @param mixed $extra - variable allowing you to pass ay data needed to the gateway, could be things like addresses that are not tracked by the transactions model
	 */
	public function setupPayment($transactionId, $return, $model, $amount, $items, $extra = array()) {
		$this->_params = [];
		$this->Transaction->id = $transactionId;

		if (is_array($amount)) {
			$currency = $amount['currency'];
			$amount = $amount['amount'];
		} else {
			$currency = Configure::read('EvTransactions.currency');
		}

		$source = !empty($this->_controller->request->data['stripeToken']) ? $this->_controller->request->data['stripeToken'] :null;

		$this->_params = compact('amount', 'currency', 'source');
	}

	/**
	 * getPayment
	 *
	 * Everything should be setup, actually take the payment
	 *
	 * @param int transactions id
	 * @return mixed dependent on the gateway, value is return straight from the transaction component to user anyway
	 */
	public function getPayment($transactionId) {
		try {
			$response = $this->_api->purchase($this->_params)->send();
			if ($response->isSuccessful()) {
				// The payment has been successfully taken, so store the payment ID.
				$data = [
					'id' => $transactionId,
					'status' => 'success',
					'message' => $response->getMessage(),
					'payment_token' => $response->getData()['id']
				];
			} else {
				// Payment has failed.
				$data = [
					'id' => $transactionId,
					'status' => 'failed',
					'message' => $response->getMessage()
				];
			}
			$this->Transaction->save($data);
		} catch (\Exception $e) {
			debug($e);
			throw new InternalErrorException(
				__d('ev_stripe', 'Sorry, there was an error processing your payment. Please try again later.')
			);
		}

		// Unlike most payment gateways that would now be sending you off to their site for payment
		// and redirecting back to a callback we have already processed the payment. Therefore, we
		// return the status of the payment here rather than via `processReturn()`.
		return [
			'result' => $response->isSuccessful(),
			'message' => $response->getMessage(),
			'transaction_id' => $transactionId
		];
	}

	/**
	 * Stripe doesn't need this as we've already processed the return at the point of getting the
	 * payment. We need to leave this here due to the interface we're implementing.
	 */
	public function processReturn() {
	}

}
